#!/usr/bin/env bash
usage() { echo -e "Usage: \
$0 \n [-n Name of the backup folder] \n \
[-s Source folder] \n \
[-t Target folder] \n \
[-r SSH options (optional)] \n \
[-e Email (optional)] \n \
[-u MySQL Username (optional)] \n \
[-p MySQL Password (optional)] \n \
[-h Host name (optional)] \n" \
1>&2; exit 1; }

while getopts "n:o::s:t:r::e::p::u::h" o; do
    case "${o}" in
        n)
            BACKUP_NAME=${OPTARG}
            ;;
        o)
            HOST=${OPTARG}
            #echo $HOST
            ;;
        s)
            SRC=${OPTARG}
            #echo $SRC
            ;;            
        t)
            BACKUP_REPO=${OPTARG}
            #echo $BACKUP_REPO
            ;;                        
        r)
            SSH_COMMAND=${OPTARG}
            #echo $SSH_COMMAND
            ;;                        
        e)
            EMAIL=${OPTARG}
            ;;                        
        u)
            MYSQL_USER=${OPTARG}
            #echo $MYSQL_USER
            ;;                        
        p)
            MYSQL_PASSWORD=${OPTARG}
            #echo $MYSQL_PASSWORD
            ;;              
        h)
            usage
            ;;   
        *)
            usage
            ;;
    esac
done

echo "Starte Backup..."
#Backup name
#n
if [ -z "$BACKUP_NAME" ]; then
 echo "Fehler: Kein Backup-Name angegeben"
 usage
exit 1
fi
#h
# Optional hostname
#if [ -n "$2" ]; then
#    HOST=$2          # host
#else 
#    HOST=""
#fi
#s
# Source files
if [ -n "$SRC" ]; then 
  if [ -n "$HOST" ]; then 
   SRC=${HOST}:$SRC
  fi
else 
    echo "Fehler: Keine Quelle angegeben"
    exit 1
fi

# Backup location
#t
if [ -z "$BACKUP_REPO" ]; then
 echo "Fehler: Kein Backup-Ziel angegeben"
 exit 1
fi
#add /
if [ "${BACKUP_REPO:${#BACKUP_REPO}-1:1}" != "/" ]; then
  BACKUP_REPO=$BACKUP_REPO/
fi

# Optional
# Custom ssh command (like ssh -i keyfile -l remote_user)
#r
if [ -z "$SSH_COMMAND" ]; then
    SSH_COMMAND=""
fi

# Optional Email for logging
#e
#if [ -n "$6" ]; then
#    EMAIL=$6        
#fi

# Optional backup of mysql (need username and password)
#m
echo $MYSQL_USER
echo $MYSQL_PASSWORD
if [ -n "$MYSQL_USER" -a -n "$MYSQL_PASSWORD" ]; then
    echo "Mysql Backup 1"
    if [ -n "$HOST" -a -n "$SSH_COMMAND" ]; then
        $SSH_COMMAND $HOST "mysqldump -u '$HOST' -p'$MYSQL_PASSWORD' --all-databases | gzip > ~/mysql.gz"
    else
        echo "Mysql Backup"
        mysqldump -u "$MYSQL_USER" -p"$MYSQL_PASSWORD" --all-databases | gzip > ~/mysql.gz
    fi;
fi

# Standard variables
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )" 
DST="$BACKUP_REPO$BACKUP_NAME"
EXCLUDE="$DIR/$BACKUP_NAME.exclude"
INCLUDE="$DIR/$BACKUP_NAME.include"
LOG="$DIR/$BACKUP_NAME.log"
RSYNC_PATH="sudo /usr/bin/rsync"

DATE=`date +%Y-%m-%d`   # Full date 2012-12-31
DOW=`date +%w`          # Day of the week 1 is Monday
DOM=`date +%d`          # Date of the Month e.g. 27

# Create non-existing directories
if [ ! -d "$DST/current" ]; then
    mkdir -p $DST/current
fi
if [ ! -d "$DST/Daily" ]; then
    mkdir $DST/Daily
fi
if [ ! -d "$DST/Weekly" ]; then
    mkdir $DST/Weekly
fi
if [ ! -d "$DST/Monthly" ]; then
    mkdir $DST/Monthly
fi
if [  -d "$DST/incomplete" ]; then
    rm -rf $DST/incomplete
fi

# Create empty exclusion file if it not exists
if [ ! -f "$EXCLUDE" ] ; then
    touch $EXCLUDE
fi

# Create empty inclusion file if it not exists
if [ ! -f "$INCLUDE" ] ; then
    touch $INCLUDE
fi

# Monthly full backup
if [ $DOM = "01" ]; then
        DATE_DST=$DST/Monthly/`date +%B`

# Weekly full backup
elif [ $DOW = "5" ]; then
    	DATE_DST=$DST/Weekly/$DATE

# Make incremental backup - overwrite last weeks
else
	DATE_DST=$DST/Daily/`date +%A`
fi

if [ "$EMAIL" ]; then
EMAIL='cat '${LOG}' | mail -s "Rsync '${BACKUP_NAME}': success" '${EMAIL}' || cat $LOG | mail -s "Rsync '${BACKUP_NAME}': failed" '${EMAIL}''
fi

# Execute the rsync task
# 
rsync -p -v -e "$SSH_COMMAND" -az --rsync-path="$RSYNC_PATH" --numeric-ids --stats --human-readable --delete --include-from "$INCLUDE" --exclude-from "$EXCLUDE" --delete-excluded --link-dest=$DST/current $SRC $DST/incomplete
# > $LOG 2>&1 && eval $EMAIL

# Delete existing destination directory
if [ -d "$DATE_DST" ]; then
    rm -rf $DATE_DST
fi

# Move backup to destination and setup new reference directory 
mv $DST/incomplete $DATE_DST
rm -rf $DST/current 
ln -s $DATE_DST $DST/current
echo "Backup abgeschlossen"


