#!/usr/bin/env bash
#
# Fritzbox-Schalter (FRITZ!Smart Energy 200 u.ä.) per Bash-Script steuern.
# 
# Die Adresse der Fritzbox
# Beispiele:
# fbox="https://192.168.178.1:444" # SSL-Verschlüsselt, Port 444
# fbox="http://192.168.178.1" # http normal, Port 80
# fbox="abcdef28kk6oabcdef.myfritz.net:444" # myfritz-Adresse, SSL, Port 444
# fbox="https://[2001:a00:12b7:c300:a00:a000:feed:921c]:444" # IPv6-Adresse, SSL, Port 444

# Installieren Sie curl, wenn nicht vorhanden
# sudo apt installl curl

# Wenn Sie das Script ohne Parameter starten, gibt es die Liste der Schalter aus.
# Verwenden Sie die Nummer des Schalters (Actor #1, Actor #2, ...)
# zum Schalten, etwa mit
# ./smart_fritz_terminal.bash 3 on
# ./smart_fritz_terminal.bash 3 off
# ./smart_fritz_terminal.bash 3 toggle (umschalten)

# Externe IP ermitteln
# ./smart_fritz_terminal.bash IP

# Anruferliste herunterladen und in der Datei anruferliste.csv speichern
#./smart_fritz_terminal.bash ANRUF

#################
# Konfiguration #
#################
fbox="http://192.168.178.1" # Bitte Adresse eintragen
# Nur bei https. Ausnahme für selbst signiertes Zertifikat, sonst leer lassen.
#INSECURE="--insecure"
INSECURE=  
# Benutzername und Passwort für die Fritzbox.
# Dieser lässt sich unter "System -> FRITZ!Box-Benutzer" erstellen.
# Dann die Option
# "Anmeldung mit FRITZ!Box-Benutzernamen und Kennwort" aktivieren
USER="DerBenutzer"
PASSWD="DasPasswort"

# Vollständige Anzeige: full, leer lassen für schnellere Ergebnisse
#VIEW="full"
VIEW=

# List der Schalter beim ersten Aufruf in der Datei ain_list.csv speichern.
# Das beschleunigt die Verarbeitung
# Nach der Neukonfiguration der Schalter diese Datei bitte löschen.
USE_CACHE_FILE="yes"
# Keine Cache-Datei verwenden.
# USE_CACHE_FILE="no"
#####################
# Konfiguration Ende#
#####################

if [ "$fbox" = "" ]; then
echo Bitte Fritzbox Adresse im Script eintragen
exit 1
fi

RED='\033[0;31m'
GREEN='\033[0;32m'
CYAN='\033[0;36m' 
YELLOW='\033[0;33m'
NC='\033[0m'
CURL=$(which curl) 
MD5SUM=$(which md5sum)
ICONV=$(which iconv)
AWK=$(which awk)

# Die SmartHome-Seite der Fritzbox
="$CURL $INSECURE -s $fbox/webservices/homeautoswitch.lua"

# Funktion zum Ermitteln der SID / Anmeldung bei der Fritzbox
get_sid() {
FRITZLOGIN="/login_sid.lua"
TEMPFile="./temp.log"
SID=$($CURL $INSECURE -s $fbox/login_sid.lua | sed 's/.*<SID>\(.*\)<\/SID>.*/\1/')
echo $SID
if [ "$SID" = "0000000000000000" ]; then
challenge=$($CURL $INSECURE -s $fbox/login_sid.lua |  grep -o "<Challenge>[a-z0-9]\{8\}" | cut -d'>' -f 2)
echo challenge: $challenge
CPSTR="$challenge-$PASSWD"
hash=`echo -n $CPSTR | $ICONV -f ISO8859-1 -t UTF-16LE | $MD5SUM -b | $AWK '{print substr($0,1,32)}'`
echo MD5: $hash
RESPONSE="$challenge-$hash"
POSTDATA="?username=$USER&response=$RESPONSE"
GETDATA="?username=$USER&response=$RESPONSE"
SID=$($CURL $INSECURE -s -G -d "response=$RESPONSE" -d "username=$USER" $fbox/login_sid.lua | sed 's/.*<SID>\(.*\)<\/SID>.*/\1/')
fi    
echo SID: $SID
}

# Funktion zum Ermitteln von Werten
get_value() {
RESULT=""
if [ "$2" = "" ]; then
RESULT=$($CURLCMD"?sid=$SID&switchcmd=$1")
else
RESULT=$($CURLCMD"?sid=$SID&ain=$2&switchcmd=$1")
fi
}
# Alle Werte ermitteln/ausgeben (langsamer)
full_output () {
   get_value getswitchstate $1
   echo State: $RESULT
   get_value getswitchpower $1
   switchpower=`awk "BEGIN {printf \"%.2f\n\", $RESULT/1000}"`
   echo Power: $switchpower W
   get_value getswitchenergy $1
   echo Energy: $RESULT Wh
   get_value gettemperature $1
   temperature=`awk "BEGIN {printf \"%.1f\n\", $RESULT/10}"`
   echo Temperature: $temperature °C
   echo

}
# Schalter schalten
set_value() {
#Parameters: $COUNTER $AIN $2
  if [ "$3" = "on" ]; then
    get_value setswitchon $2
	echo "set #$1 on"
  fi	
    if [ "$3" = "off" ]; then
      get_value setswitchoff $2
	  echo "set #$1 off"
    fi	
    if [ "$3" = "toggle" ]; then
      get_value setswitchtoggle $2
	  echo "set #$1 toggle"
    fi	
}

# Externe IP ermitteln, kein Login erforderlich
if [ "$1" == "IP" ]; then
echo "Externe IP ermitteln"
IP=$(curl -s "$fbox:49000/igdupnp/control/WANIPConn1" \
-H "Content-Type: text/xml; charset="utf-8"" \
-H "SoapAction:urn:schemas-upnp-org:service:WANIPConnection:1#GetExternalIPAddress" \
-d '<?xml version="1.0" encoding="utf-8"?><s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"> <s:Body> <u:GetExternalIPAddress xmlns:u="urn:schemas-upnp-org:service:WANIPConnection:1" /></s:Body></s:Envelope>' | \
  grep -Eo '\<[[:digit:]]{1,3}(\.[[:digit:]]{1,3}){3}\>')
echo "Die aktuelle externe IP ist $IP"
exit 0
fi

############ Script Start ################
get_sid # SID holen

if [ "$SID" = "0000000000000000" ]; then
echo -e  "${RED}Anmeldung fehlgeschlagen ${NC}"
exit 1
fi
if [ "$SID" = "" ]; then
echo -e  "${RED}Anmeldung fehlgeschlagen ${NC}"
exit 1
fi

echo -e  "${GREEN}Anmeldung erfolgreich ${NC}"
echo

# Anrufliste herunterladen
if [ "$1" == "ANRUF" ]; then
CALLERLIST="./anruferliste.csv"
get_sid
wget -O - "$fbox/fon_num/foncalls_list.lua?sid=$SID&csv=" >$CALLERLIST 
exit 0
fi
# Schalter ermitteln oder schalten
COUNTER=0

if [ "$USE_CACHE_FILE" == "yes" ]
then
if [ -e ain_list.csv ]
then
echo "Liste aus der Cache-Datei ain_list.csv"
RESULT=`cat ain_list.csv`
IFS=', ' read -r -a array <<< "$RESULT"
for AIN in "${array[@]}"
do
let COUNTER=COUNTER+1 

echo $COUNTER $AIN
if [ "$1" = "$COUNTER" ]
then
set_value $COUNTER $AIN $2 
fi
if [ "$VIEW" = "full" ]
then
full_output $AIN
fi
done
fi
fi

if [ ! -e ain_list.csv ]
then
# Liste der Schalter ermitteln
get_value getswitchlist

if [ ! "$USE_CACHE_FILE" == "no" ]
then
echo "Estelle Cache-Datei ain_list.csv"
echo $RESULT >> ain_list.csv
fi
IFS=', ' read -r -a array <<< "$RESULT"
# Werte der Schalter holen
for AIN in "${array[@]}"
do
let COUNTER=COUNTER+1 
echo -e  "${YELLOW}Actor #$COUNTER ${NC}"
switchpresent=0
get_value getswitchname $AIN
echo -e  "${CYAN}Name: $RESULT ${NC}"
get_value getswitchpresent $AIN
echo AIN: $AIN
echo Connected: $RESULT
if [ "$RESULT" = "1" ]; then
# bei aktiven Schaltern 
# Parameter verarbeiten, etwa: 1 on
if [ "$1" = "$COUNTER" ]; then
set_value $COUNTER $AIN $2 
fi
fi
if [ "$VIEW" = "full" ]
then
full_output $AIN
fi
done
fi




